      DOUBLE PRECISION FUNCTION GAMMA (F,N,M)
C Tide generating potential * NDR spectrum
C
C Computes the gamma-factor for the tide raising potential w.r.t. frequency.
C Frequency dependent feature included: Near diurnal wobble.
C
C The resulting values are to be used with secondary effects; the values
C do not consider ellipticity nor Eulerian potential.
C
C The values come from Farrell, 1972, except for n=2 where the Basic Love
C numbers from COMMON /EPAR/: BL(1) and BL(3) are used
C (for m=1: resonance strength parameters from COMMON /COREP/: H1 and K1),
C and degrees 4 and 5 using Mitrovica's program:
C                            h              l              k
C    2      Elastic:     6.05282D-01    8.43286D-02    2.99315D-01
C    3      Elastic:     2.89136D-01    1.48066D-02    9.24685D-02
C    4      Elastic:     1.75701D-01    1.02192D-02    4.16112D-02
C    5      Elastic:     1.29421D-01    8.46903D-03    2.44329D-02
C    6      Elastic:     1.07406D-01    6.79348D-03    1.68537D-02
C
C Calling parameters:
C
C F   real*8   Frequency in rad/h
C N   integer  Spherical harmonic degree of the tide. 2 <= N <= 5
C M   integer      "        "     order  "   "   "    0 <= M <= N
C
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      REAL*8 K0,K1,L0,L1
      COMMON /COREP/ FNDR,Q,F0,H1,L1,K1,ZOH,ZOL,ZOK
      COMMON /EPAR/  R0,RE,GE,EROT,H0,L0,K0,E1,E2,E3,E4,E5,E6
      COMMON /MPAR/  PI,RAD
      SAVE
      IF (N.LE.1) THEN
         PRINT '(A,I5)',' <GAMMA->E> Illegal sph.harm.degree:',N
         G=0.0D0
      ELSE IF (N.GT.5) THEN
         PRINT '(A,I5)'
     &,' <GAMMA->A> Apology: I don''t know 1+kn-hn for n=',N
         G=0.0D0
      ELSE IF (N.EQ.5) THEN
         G=1.D0+0.024D0-0.129D0
      ELSE IF (N.EQ.4) THEN
         G=1.D0+0.0416D0-0.1757D0
      ELSE IF (N.EQ.3) THEN
         G=1.D0+0.0943D0-0.2913D0
      ELSE IF (N.EQ.2) THEN
         IF (M.EQ.1) THEN
            F1=(1.D0+1.D0/FNDR)*EROT
            G=1.D0+K0-H0+(K1*K0-H1*H0)*(F-F0)/(F-F1)
         ELSE
            G=1.D0+K0-H0
         ENDIF
         IF (DABS(G).GT.1.5D0.OR.DABS(G).LT..1D0) THEN
            PRINT '(A,F10.6,A,1P,D12.4)',' <GAMMA->W> Great anomaly at '
     &,     F/RAD,' deg/h: Gamma = ',G
         ENDIF
      ENDIF
      GAMMA=G
      RETURN
      END

      COMPLEX*16 FUNCTION ZGAMMA (F,N,M)
c
c That's another, slightly improved but still not complete account of
c the Love number combination for the effective ocean tide generating 
c potential. It is applicable as a global mean; the latitudinal dependence 
c is unaccounted.
c
c This version includes the finite quality of the resonance and the ocean
c load excited resonance. 
c 
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      REAL*8   K0,K1,L0,L1
     &,        CORDH(2:4,0:4), CORDL(2:4,0:4), CORDK(2:4,0:4)

      COMMON /COREP/  FRENDR,QNDR,F0,H1,L1,K1,ZOH,ZOL,ZOK
      COMMON /EPAR/   R0,RE,GE,EROT,H0,L0,K0,E1,E2,E3,E4,E5,E6
      COMMON /MPAR/   PI,RAD
      COMMON /SHDOP/  CORDH, CORDL, CORDK

      SAVE

      FNDR=(1.D0+1.D0/FRENDR)*EROT
      ZFNDR=CMPLX(FNDR,FNDR/QNDR/2.D0)
      IF (N.LE.1) THEN
         PRINT '(A,I5)',' <ZGAMMA>E> Illegal sph.harm.degree:',N
         ZG=(0.0D0,0.0D0)
      ELSE IF (N.GT.5) THEN
         PRINT '(A,I5)'
     &,' <ZGAMMA>A> Apology: I don''t know 1+kn-hn for n=',N
         ZG=(0.0D0,0.0D0)
      ELSE IF (N.EQ.5) THEN
         ZG=DCMPLX(1.D0+0.024D0-0.129D0,0.D0)
      ELSE IF (N.EQ.2.AND.M.EQ.1) THEN
         ZG=(1.D0,0.0D0)+K0-H0
     &     +((K1*K0-H1*H0)*(F-F0)+(K1*ZOK-H1*ZOH)*EROT)/(F-ZFNDR)
         IF (ABS(ZG).GT.1.5D0.OR.ABS(ZG).LT..1D0) THEN
            PRINT '(A,F10.6,A,1P,2D12.4)'
     &,           ' <ZGAMMA>W> Great anomaly at '
     &,             F/RAD,' deg/h: Gamma = ',ZG
         ENDIF
      ELSE IF ((N.EQ.2.AND.M.NE.1).OR.N.EQ.3.OR.N.EQ.4) THEN
         ZG=(1.D0,0.0D0)+CORDK(N,M)*K0-CORDH(N,M)*H0
      ENDIF
      ZGAMMA=ZG
      RETURN
      END
