C     ------------------------------------------------      
      COMPLEX*16 FUNCTION ZPNM (N,M,A,B)
C     ------------------------------------------------
C Spherical harmonic, P(N,M,COLATITUDE,LONGITUDE)
C angles in RAD
C>
C We refer to these functions occasionally (e.g. in ~/Thtide/p/astros.f)
C as "Conventional Associated Legendre functions".
C However, observe that this only charaterizes the (lack of unit- or so)
C normalisation, not the sign that originallys goes as (-1)**m
C 
C cf http://mathworld.wolfram.com/AssociatedLegendrePolynomial.html
C In the perspective of mathworld, we compute here the
C Abramovitz-Stegun functions P_{nm}, not P_n^m
C
C In the second part of this file we use recurrence relations to
C compute the polynomial part of Y_n^m (and internally the
C conventional P_n^m) for small degrees. 
C
C If you need Fully Normalized Associated Legendres up to high degrees, use 
c math/afor/p/alpfn.f
C_ 
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON /MPAR/ PI,RAD
      DATA ZI/(0.0d0,1.0d0)/

      ZPNM = PNM (N,M,A) * EXP(ZI*M*B)
      RETURN
      END
C
C     -------------------------------------
      DOUBLE PRECISION FUNCTION PNM (N,M,A)
C     -------------------------------------
C Spherical harmonics * PNM(cos(a)) for low n,m   (beware n <= 4)
C>                                                      m
C  Conventional Associated Legendre Polynomials PNM = Pn (cos A).
C  Explicit forms for low degrees.
C
C  N = degree, M = order             -N <= M <= N
C  A = angle (rad)                    0 <= A <= PI with RAD as a grace margin
C
C  Entry PNMA (N,M,X)                -1 <= X <= 1  strictly
C_
      IMPLICIT REAL*8 (A-H,O-Z)
      LOGICAL QA
      COMMON /MPAR/ PI,RAD
      DATA ierr/0/
      PNM=0.0D0
      X=DCOS(A)
      Y=DSIN(A)
      QA=.FALSE.
      GOTO 1

      ENTRY PNMA (N,M,A)
C Spherical harmonics * PNM(A) for low n,m
C
      PNMA=0.0D0
      QA=.TRUE.
      X=DMIN1(1.D0,DMAX1(-1.D0,A))
      Y=DSQRT(1.D0-X*X)

1     CONTINUE
      IF (A.LT.-RAD.OR.A.GT.PI+RAD) THEN
         PRINT '(A,1P,D12.5,A)',' <PNM--->E> Argument out of range: '
     &    ,A/RAD,' (deg) '
         PRINT '(A,1P,2D12.5)',' <PNM--->I> PI, RAD = ',PI,RAD
         RETURN
      ENDIF
      IF (N.GT.4.OR.N.LT.0) THEN
         ierr=ierr+1
         if (ierr.lt.11) 
     &       PRINT '(A,I5)',' <PNM--->E> Degree not avail.: ',N
         RETURN
      ENDIF
      IM=IABS(M)
      IF (IM.GT.N) RETURN
      X2=X*X
      Y2=Y*Y
      GOTO (100,101,102,103,104), N+1
100   P=1.0D0
      GOTO 201

101   IF (IM.EQ.0) THEN
         P=X
      ELSE
         P=Y
      ENDIF
      GOTO 201
102   IF (IM.EQ.0) THEN
         P=(3.D0*X2-1.D0)/2.D0
      ELSE IF (IM.EQ.1) THEN
         P=3.D0*X*Y
      ELSE
         P=3.D0*Y2
      ENDIF
      GOTO 201
103   IF (IM.EQ.0) THEN
         P=(5.D0*X2-3.D0)*X/2.D0
      ELSE IF (IM.EQ.1) THEN
         P=(5.D0*X2-1.D0)*Y*1.5D0
      ELSE IF (IM.EQ.2) THEN
         P=15.D0*Y2*X
      ELSE
         P=15.D0*Y2*Y
      ENDIF
      GOTO 201
104   IF (IM.EQ.0) THEN
         P=(35.D0*X2*X2-30.D0*X2+3.D0)/8.D0
      ELSE IF (IM.EQ.1) THEN
         P=(7.D0*X2-3.D0)*X*Y*2.5D0
      ELSE IF (IM.EQ.2) THEN
         P=(7.D0*X2-1.D0)*Y2*7.5D0
      ELSE IF (IM.EQ.3) THEN
         P=105.D0*Y2*Y*X
      ELSE
         P=105.D0*Y2*Y2
      ENDIF

201   IF (QA) GOTO 211
      PNM=P
      IF (M.GE.0) RETURN
      FAC=1.D0
      DO 203 I=N-IM+1,N+IM
203   FAC=FAC*DFLOAT(I)
      PNM=P/FAC
      RETURN

211   PNMA=P
      IF (M.GE.0) RETURN
      FAC=1.D0
      DO 213 I=N-IM+1,N+IM
213   FAC=FAC*DFLOAT(I)
      PNMA=P/FAC
      RETURN
      END
C
C
C     -----------------------------
      real*8 function dpnma (n,m,y)
C Spherical harmonics * dPNM(cos(y))/dy for low n,m
C     -----------------------------
C>
C     Derivative of PNM w r t angle A (=Y here), using interpolation
C     polynomial w r t arccos, more accurate than DPNM.
C_
      implicit real*8 (a-h,o-z)
      common /mpar/ pi,rad
      dimension x(-2:2),p(-2:2),cof(0:4)
      z=dcos(y)
      zs=dmin1(0.9d0,dmax1(-0.9d0,z))
      do 101 i=-2,2
      zv=zs+dfloat(i)/20.d0
      x(i)=zv
101   p(i)=pnma(n,m,zv)
      call polcoe (x,p,5,cof)
      dv=cof(4)*4.d0
      do 102 i=3,1,-1
102   dv=dv*z+cof(i+1)*dfloat(i)
      dpnma=-dv*dsin(y)
      return
      end
C
C     ----------------------------
      real*8 function dpnm (n,m,y)
C Spherical harmonics * dPNM(cos(y))/dy for low n,m; less accurate
C     ----------------------------
C>
C    Derivative of PNM w r t angle A, using interpolation polynomial
C_
      implicit real*8 (a-h,o-z)
      common /mpar/ pi,rad
      dimension x(-2:2),p(-2:2),cof(0:4)
      do 101 i=-2,2
      xv=y+dfloat(i)*rad/2.d0
      x(i)=xv
101   p(i)=pnm(n,m,xv)
      call polcoe (x,p,5,cof)
      dv=cof(4)*4.d0
      do 102 i=3,1,-1
102   dv=dv*y+cof(i)*dfloat(i)
      dpnm=dv
      return
      end
C
C
C     -----------------------------
      SUBROUTINE POLCOE (X,Y,N,COF)
C Interpolation polynomial *  Numerical Recipes
C     -----------------------------
C
      implicit real*8 (a-h,o-z)
      PARAMETER (NMAX=15)
      DIMENSION X(N),Y(N),COF(N),S(NMAX)
      DO 11 I=1,N
        S(I)=0.d0
        COF(I)=0.d0
11    CONTINUE
      S(N)=-X(1)
      DO 13 I=2,N
        DO 12 J=N+1-I,N-1
          S(J)=S(J)-X(I)*S(J+1)
12      CONTINUE
        S(N)=S(N)-X(I)
13    CONTINUE
      DO 16 J=1,N
        PHI=dfloat(N)
        DO 14 K=N-1,1,-1
          PHI=dfloat(K)*S(K+1)+X(J)*PHI
14      CONTINUE
        FF=Y(J)/PHI
        B=1.d0
        DO 15 K=N,1,-1
          COF(K)=COF(K)+B*FF
          B=S(K)+X(J)*B
15      CONTINUE
16    CONTINUE
      RETURN
      END
c
c Some factorials for small integers
c
c     -----------------------------
      integer function nfactrls (n)
c     -----------------------------
c
c Factorial of small n
      i=max0(1,n)
      do j=n-1,2,-1
         i=i*j
      enddo
      nfactrls=i
      return
      end
c
c     ---------------------------------
      integer function nfactrls_2n1 (n)
c     ---------------------------------
c
c Double-factorial of 2n-1 for small n
c     i=2*n-1
      do j=n-1,1,-1
         i=i*(2*j-1)
      enddo
      nfactrls_2n1=i
      return
      end
c
c     ------------------------
      subroutine ywnm_init (n)
c     ------------------------
c
c Prepares a table of prefactors for Ynm = YWnm Pnm exp(i m x)
c n <= 6
c
      implicit real*8 (a-h,o-z)
      parameter (nmax=6)
      dimension ywnm(0:nmax,0:nmax)
      logical qi,qy,qp
      common /cywnm/ ywnm, qi,qy,qp
      common /mpar/  pi,rad
      
      pi=4.d0*datan(1.0d0)
      rad=pi/180.d0

      if (n.lt.0.or.nt.gt.nmax) then
         print *,'<YWInit>T> Illegal degree ',n,' the maximum is ',nmax
         stop
      endif

      ywnm(0,0)=1.d0

      do i=1,n
         ywnm(i,0)=dfloat(2*i+1)/4/pi
      enddo
      do i=1,n
         do j=1,i
            ywnm(i,j)=ywnm(i,j-1)/dfloat(i-j+1)/dfloat(i+j)
         enddo
      enddo
      ywnm(0,0)=dsqrt(ywnm(0,0))
      do i=1,n
         do j=1,i
            ywnm(i,j) = (-1)**j *dsqrt(ywnm(i,j))
         enddo
      enddo
      qi=.true.
      if (qp) then
         do i=0,n
            print *,'<ywnm-->>> ',i,(ywnm(i,j),j=0,i)
         enddo
      endif
      return
      end

c ------------------------------------------------- Ynm part
c     --------------------------
      real*8 function ynm(n,m,a)
c     --------------------------
c
c The Ynm(cos(a)) function's polynomial part, n <= 6 by recurrence
c
c Parameter NMAX controls maximum degree.
c
c As soon as <a> changes, the recurrence loop is activated.
c If you want to watch the results, call ynm_init (.true.)
c
c If you need Ynm(a), use the entry ynma(n,m,a)
c
c If you need the Pnm's, add
c
c     COMMON /CYWNM/ YWMN(0:NMAX,0:NMAX)
c 
c to your program and compute Pnm = Ynm(n,m,a)/YWnm(n,m)
c
c Derivatives w.r.t. a: Use entries dYnm(n,m,a)  or  dYnma(n,m,a)
c
      implicit real*8 (a-h,o-z)
      parameter (nmax=6)
      dimension ywnm(0:nmax,0:nmax),aynm(0:nmax,0:nmax)
      logical qi,qa,qy,qp,qd
      common /cywnm/ ywnm, qi,qy,qp

      data xold/-9999.9d9/

      x=dcos(a)
      y=-dsin(a)
      qa=.false.
      qd=.false.
      goto 101
c
c     -----------------
      entry ynma(n,m,a)
c     -----------------
      qa=.true.
      qd=.false.
      x=a
      y=-dsqrt(1.d0-x*x)
      goto 101
c
c     -----------------
      entry dynm(n,m,a)
c     -----------------
      qd=.true.
      qa=.false.
      x=dcos(a)
      y=-dsin(a)
      goto 101
c
c     ------------------
      entry dynma(n,m,a)
c     ------------------
      qd=.true.
      qa=.true.
      x=a
      y=-dsqrt(1.d0-x*x)

101   if (qp) print *,'<ynm--->d> qi=',qi
      if (.not.qi) call ywnm_init (nmax)

      qy=qy.or.dabs(xold-x).gt.1d-10
      if (qp) print *,'<ynm--->d> qy=',qy
      xold=x
      if (qy) then
         aynm(0,0)=1.d0
         aynm(1,0)=x
         aynm(1,1)=-y
      
         do i=2,nmax
            aynm(i,0)=(x*(2*i-1)*aynm(i-1,0)
     &                -(i-1)*aynm(i-2,0))/dfloat(n)
         enddo
c
c Do this only if |y| > eps
c else set all aymn(i,j)=0.0 for j > 0
c
         if (dabs(y).lt.1.d-10) then
            do i=2,nmax
               do j=1,i
                  aynm(i,j)=0.0d0
               enddo
            enddo
         else
          do i=2,nmax-1
            aynm(i,1)=( i*(i+1)*aynm(i+1,0) -i*(i+1)*aynm(i-1,0) )/y
          enddo
          aynm(nmax,nmax)=(-1)**nmax *y**nmax * nfactrls_2n1(nmax)

          do i=2,nmax
           do j=1,i-1
            aynm(i,j+1)=((i-j)*x*aynm(i,j)-(i+j)*aynm(i-1,j))/y
           enddo 
          enddo
          qy=.false.
          if (qp) then
            do i=0,nmax
              print *,' '
              print '(a,i2,1p,7e12.4)'
     &,             ' <Ynm--->>> f ',i,(aynm(i,j),j=0,i)
              print '(a,i2,1p,7e12.4)'
     &,             ' <Ynm--->>> w ',i,(ywnm(i,j),j=0,i)
            enddo
          endif
         endif
      endif
      if (qd) then
c The derivative w.r.t. a 
         if (qa) then
            if (n.le.0.or.dabs(y).lt.1.d-10) then
               dynma=0.d0
            else
               dynma=-(n*x*aynm(n,m)-(n+m)*aynm(n-1,m))/y/y*ywnm(n,m)
            endif
         else
c The derivative of Ynm(cos(a)) w.r.t. a
            if (n.le.0.or.dabs(y).lt.1.d-10) then
               dynma=0.d0
            else
               dynm=(n*x*aynm(n,m)-(n+m)*aynm(n-1,m))/y*ywnm(n,m)
            endif
         endif
      else
c The function value
         if (qa) then
            ynma=aynm(n,m)*ywnm(n,m)
         else
            ynm=aynm(n,m)*ywnm(n,m)
         endif
      endif
      return
      end
c
c     ------------------------
      subroutine ynm_init (qq)
c     ------------------------
c 
c Requests re-computation and sets the print option
c
      implicit real*8 (a-h,o-z)
      parameter (nmax=6, n=nmax+1, nn=n*n)
      dimension ywnm(0:nmax,0:nmax)
      logical qi, qy, qp, qq
      common /cywnm/ ywnm, qi,qy,qp
      qy=.true.
      qp=qq
      return
      end
c
c     ----------------
      block data bywnm
c     ----------------
      implicit real*8 (a-h,o-z)
      parameter (nmax=6, n=nmax+1, nn=n*n)
      dimension ywnm(0:nmax,0:nmax)
      logical qi, qy, qp
      common /cywnm/ ywnm, qi, qy, qp
      data ywnm/nn*0.0d0/, qi,qp/2*.false./, qy/.true./
      end
