C Subroutines for vector rotation / coordinate ref.sys. rotation
C (Routines prefixed by ZV_ or V_: take explicitly specified vector components
C as arguments.)
C
C ROTATE:
C RIN(3)    - real*8      - Input cartesian vector
C ROUT(3)   - real*8      - Output ...
C                           Rotation is with currently defined matrix
C
C Z_ROTATE:
C RIN(3)    - complex*16  - Input complex cartesian vector
C ROUT(3)   - complex*16  - Output ...
C                           Rotation is with currently defined matrix
C
C V_ROTATE:
C VI1..VI3  - real*8      - Input cartesian vector elements
C VO1..VO3  - real*8      - Output ...
C                           Rotation is with currently defined matrix
C
C ZV_ROTATE:
C ZI1..ZI3  - complex*16  - Input complex cartesian vector elements
C ZO1..ZO3  - complex*16  - Output ...
C                           Rotation is with currently defined matrix
C
C EULROT:
C IAX       - integer     - Number of axis for rotation
C ANG       - real*8      - Rotation angle (rad)
C                           The routine updates the current rotation matrix
C                           by left-multiplication with the new rot.-mat.
C
C RESET_ROT:
C                           Resets current rotation matrix to unit matrix.
C
C GET_ROT:
C GM(3,3)   - real*8      - Currently defined rotation matrix, returned.
C
C SET_ROT:
C GM(3,3)   - real*8      - Explicitly specify a rotation matrix.
C
C SHOW_ROT:
C COMMENT   - char*(*)    - Print currently defined rotation matrix; 
C                           comment is printed in headline
C
C SET_LOCAL_AT:
C xlon,xcolat - real*8    - set local point by geocentric position (radians!)
C
C SET_CHORD_TO:
C ylon,ycolat - real*8    - set remote point by geocentric position (radians!)
C
C SET_LOCAL_AT_XYZ:
C XYZ(3)      - real*8    - set local point by cartesian geocentric position.
C 
C SET_BASELINE:
C XYZ_LOCAL(3) - real*8   
C XYZ_FAR(3)   - real*8   - local and far point cartesian geocentric position.
C
C SWAP_LOCAL
C isw          - integer  - <= 0: swap, 1: swap to normal, 2: swap to reverse
C
C SET_LTU_ROT:            - compute rotation matrix for XYZ->LTU (hidden)
C                           for a baseline
C
C SET_REN_ROT:            - compute rotation matrix for XYZ->REN (hidden) 
C                           for a single point
C
C USAGE
C To use these routines for transformation between local and geocentric
C representations, you can for instance
C    CALL SET_BASELINE (XYZ_LOCAL, XYZ_FAR)
C    CALL SET_LTU_ROT
C    CALL Z_ROTATE (Z_DELTA_XYZ, Z_DELTA_LTU)
C to get a displacement vector in components LTU with respect to the
C baseline between point "local" and point "far".
C
C    CALL LOCAL_AT (XLON_LOCAL, XCOLAT_LOCAL)
C    CALL CHORD_TO (XLON_FAR,   XCOLAT_FAR  )
C    CALL SET_LTU_ROT
C    CALL Z_ROTATE (Z_DELTA_XYZ, Z_DELTA_LTU)
C
C To get the transformed vector in REN representation,
C    CALL LOCAL_AT_XYZ (XYZ_LOCAL)
C    CALL SET_REN_ROT
C    CALL Z_ROTATE (Z_DELTA_XYZ, Z_DELTA_REN)
C    
C or 
C    CALL LOCAL_AT_XYZ (XYZ_LOCAL)
C    CALL Z_GEOC_TO_LOCAL (Z_DELTA_XYZ, Z_DELTA_REN)
C
C To define a baseline with varying far point,
C    CALL LOCAL_AT_XYZ (XYZ_LOCAL)
C    CALL SWAP_LOCAL (2)
C    CALL LOCAL_AT_XYZ (XYZ_FAR)
C
C 
C
C Subroutines for vector transformation,  local <-> geocentric reference
C without using the hidden rotation matrix:
C
C ZV_Local_to_Geoc    Arguments = explicitly specified (not: dimensioned)
C ZV_Geoc_to_Local    Complex*16
C
C V_Local_to_Geoc     Arguments = explicitly specified (not: dimensioned)
C V_Geoc_to_Local     Real*8
C
C Z_Local_to_Geoc     DIM(arguments)=3, complex*16
C Z_Geoc_to_Local
C
C Local_to_Geoc       DIM(arguments)=3, real*8
C Geoc_to_Local
C
C Local_at            Set site coordinates
C
C Description cf. below.
C
      SUBROUTINE ROTATE (RIN,ROUT)
C Utility * Perform vector rotation
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      REAL*8 CM(3,3),RIN(3),ROUT(3)
      COMMON /CROTMA/ CM
      DO 101 I=1,3
      R=0.0D0
      DO 102 J=1,3
102   R=R+CM(I,J)*RIN(J)
101   ROUT(I)=R
      RETURN
      END

      SUBROUTINE ROTATE_I (RIN,ROUT)
C Utility * Perform inverse vector rotation
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      REAL*8 CM(3,3),RIN(3),ROUT(3)
      COMMON /CROTMA/ CM
      DO 101 I=1,3
      R=0.0D0
      DO 102 J=1,3
102   R=R+CM(J,I)*RIN(J)
101   ROUT(I)=R
      RETURN
      END

      SUBROUTINE EULROT (IAX,ANG)
C Utility * Define axis and angle for Euler rotation
C
C Rotation matrix is built up successively
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      REAL*8 ANG,CM(3,3),RM(3,3),HM(3,3),GM(3,3)
      CHARACTER CMMNT*(*)
      COMMON /CROTMA/ CM
      DO 201 I=1,3
      RM(I,IAX)=0.0D0
201   RM(IAX,I)=0.0D0
      RM(IAX,IAX)=1.0D0
      DO 202 I=IAX+1,IAX+2
      II=MOD(I-1,3)+1
202   RM(II,II)=DCOS(ANG)
      JP=MOD(IAX,3)+1
      JM=MOD(IAX+1,3)+1
      RM(JP,JM)=DSIN(ANG)
      RM(JM,JP)=-RM(JP,JM)
      DO 203 I=1,3
      DO 203 J=1,3
      R=0.0D0
      DO 204 K=1,3
204   R=R+RM(I,K)*CM(K,J)
203   HM(I,J)=R
      DO 205 I=1,3
      DO 205 J=1,3
205   CM(I,J)=HM(I,J)
      RETURN

      ENTRY RESET_ROT
C Utility * Reset rotation matrix
      DO 211 I=1,3
      DO 211 J=1,3
211   CM(I,J)=0.0D0
      DO 212 I=1,3
212   CM(I,I)=1.0D0
      RETURN

      ENTRY GET_ROT (GM)
C Utility * Return rotation matrix
      DO 221 I=1,3
      DO 221 J=1,3
221   GM(I,J)=CM(I,J)
      RETURN

      ENTRY SET_ROT (GM)
C Utility * Set rotation matrix explicit
      DO 222 I=1,3
      DO 222 J=1,3
222   CM(I,J)=GM(I,J)
      RETURN

      ENTRY ADD_ROT (GM)
      DO 225 I=1,3
         DO 225 J=1,3
            HM(I,J)=0.
            DO 225 K=1,3
               HM(I,J)=HM(I,J)+GM(I,K)*CM(K,J)
 225        CONTINUE
      DO 226 I=1,3
         DO 226 J=1,3
 226        CM(I,J)=HM(I,J)
      RETURN

      ENTRY ADD_ROT_I (GM)
      DO 227 I=1,3
         DO 227 J=1,3
            HM(I,J)=0.
            DO 227 K=1,3
               HM(I,J)=HM(I,J)+GM(K,I)*CM(K,J)
 227        CONTINUE
      DO 228 I=1,3
         DO 228 J=1,3
 228        CM(I,J)=HM(I,J)
      RETURN

      ENTRY INV_ROT
      DO 223 I=1,3
      DO 223 J=1,I-1
      V=CM(I,J)
      CM(I,J)=CM(J,I)
223   CM(J,I)=V
      RETURN

      ENTRY SHOW_ROT (CMMNT)
C Utility * Print rotation matrix
      IF (INDEX_L_NONBLANK(CMMNT).GT.1)
     &   PRINT '(2A)','0<xx_ROT>>> ',CMMNT
      DO 232 J=1,3
232   PRINT '(A,1P,3D13.5)',' <xx_ROT>>> ',(CM(I,J),I=1,3)
      PRINT '(A)',' <xx_ROT>>>'
      RETURN
      END

      SUBROUTINE Z_ROTATE (RIN,ROUT)
C Utility * Rotate complex vector
      IMPLICIT REAL*8 (A-H,O-Q,S-Z), COMPLEX*16 (R)
      DIMENSION CM(3,3),RIN(3),ROUT(3)
      COMMON /CROTMA/ CM
      DO 101 I=1,3
      R=(0.0D0,0.0D0)
      DO 102 J=1,3
102   R=R+CM(I,J)*RIN(J)
101   ROUT(I)=R
      RETURN
      END

      SUBROUTINE Z_ROTATE_I (RIN,ROUT)
C Utility * Inv rotate complex vector
      IMPLICIT REAL*8 (A-H,O-Q,S-Z), COMPLEX*16 (R)
      DIMENSION CM(3,3),RIN(3),ROUT(3)
      COMMON /CROTMA/ CM
      DO 101 I=1,3
      R=(0.0D0,0.0D0)
      DO 102 J=1,3
102   R=R+CM(J,I)*RIN(J)
101   ROUT(I)=R
      RETURN
      END

      SUBROUTINE ZV_ROTATE (Zi1,zi2,zi3, Zo1,zo2,zo3)
C Utility * Rotate complex vector (components)
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      DIMENSION  CM(3,3),ZIN(3),ZOUT(3)
      COMMON /CROTMA/ CM
      zin(1)=zi1
      zin(2)=zi2
      zin(3)=zi3
      DO 101 I=1,3
      ZR=(0.0D0,0.0D0)
      DO 102 J=1,3
102   ZR=ZR+CM(I,J)*ZIN(J)
101   ZOUT(I)=ZR
      zo1=zout(1)
      zo2=zout(2)
      zo3=zout(3)
      RETURN
      END

      SUBROUTINE ZV_ROTATE_I (Zi1,zi2,zi3, Zo1,zo2,zo3)
C Utility * Inv rotate complex vector (components)
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      DIMENSION  CM(3,3),ZIN(3),ZOUT(3)
      COMMON /CROTMA/ CM
      zin(1)=zi1
      zin(2)=zi2
      zin(3)=zi3
      DO 101 I=1,3
      ZR=(0.0D0,0.0D0)
      DO 102 J=1,3
102   ZR=ZR+CM(J,I)*ZIN(J)
101   ZOUT(I)=ZR
      zo1=zout(1)
      zo2=zout(2)
      zo3=zout(3)
      RETURN
      END

      SUBROUTINE V_ROTATE (Zi1,zi2,zi3, Zo1,zo2,zo3)
C Utility * Rotate real vector (components)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION  CM(3,3),ZIN(3),ZOUT(3)
      COMMON /CROTMA/ CM
      zin(1)=zi1
      zin(2)=zi2
      zin(3)=zi3
      DO 101 I=1,3
      ZR=0.0D0
      DO 102 J=1,3
102   ZR=ZR+CM(I,J)*ZIN(J)
101   ZOUT(I)=ZR
      zo1=zout(1)
      zo2=zout(2)
      zo3=zout(3)
      RETURN
      END

      SUBROUTINE V_ROTATE_I (Zi1,zi2,zi3, Zo1,zo2,zo3)
C Utility * Inv rotate real vector (components)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION  CM(3,3),ZIN(3),ZOUT(3)
      COMMON /CROTMA/ CM
      zin(1)=zi1
      zin(2)=zi2
      zin(3)=zi3
      DO 101 I=1,3
      ZR=0.0D0
      DO 102 J=1,3
102   ZR=ZR+CM(J,I)*ZIN(J)
101   ZOUT(I)=ZR
      zo1=zout(1)
      zo2=zout(2)
      zo3=zout(3)
      RETURN
      END

      BLOCK DATA BROTMA
C Utility * Default rotation matrix
      REAL*8 CM
      COMMON /CROTMA/ CM(3,3)
      DATA CM/1.0D0,0.0D0,0.0D0,0.0D0,1.0D0,0.0D0,0.0D0,0.0D0,1.0D0/
      END

      BLOCK DATA BLOCAL
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON /CLOCAL/ X,Y,SX,CX,SY,CY,DUM2(6),DUMV(11),ISW
      DATA ISW/0/
      END

c
c The following routines transform between XYZ and REN 
c *without* accessing the rotation matrix / common area /CROTMA/ 
c 
      SUBROUTINE ZV_LOCAL_TO_GEOC (VR,VE,VN, VX,VY,VZ)
C Utility * Complex vector (components)
C
C Transform a local, complex displacement vector [V_radial, V_east, V_north]
C to geocentric reference. Local = at longitude X, colatitude Y (rad).
C The site location is set using a call to
C
C SET_LOCAL_AT (x,y)
C
      IMPLICIT REAL*8 (A-H,O-U,X,Y), COMPLEX*16 (V)
      COMMON /CLOCAL/ X,Y,SX,CX,SY,CY,DUM2(6),DUMV(11),ISW
      VTX=VR*SY*CX-VE*SX-VN*CY*CX
      VTY=VR*SY*SX+VE*CX-VN*CY*SX
      VTZ=VR*CY+VN*SY
      VX=VTX
      VY=VTY
      VZ=VTZ
      RETURN
      END

      SUBROUTINE ZV_GEOC_TO_LOCAL (VX,VY,VZ, VR,VE,VN)
C Utility * Complex vector (components)
C
C The inverse of ZV_Local_to_Geoc
C
      IMPLICIT REAL*8 (A-H,O-U,X,Y), COMPLEX*16 (V)
      COMMON /CLOCAL/ X,Y,SX,CX,SY,CY,DUM2(6),DUMV(11),ISW
      VTR=VX*SY*CX+VY*SY*SX+VZ*CY
      VTE=-VX*SX+VY*CX
      VTN=-VX*CY*CX-VY*CY*SX+VZ*SY
      VR=VTR
      VE=VTE
      VN=VTN
      RETURN
      END

      SUBROUTINE V_LOCAL_TO_GEOC (VR,VE,VN, VX,VY,VZ)
C Utility * Real vector (components)
C
C Transform a local, complex displacement vector V_radial, V_east, V_north to
C geocentric representation. Local = at longitude X, colatitude Y (rad).
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON /CLOCAL/ X,Y,SX,CX,SY,CY,DUM2(6),DUMV(11),ISW
      VTX=VR*SY*CX-VE*SX-VN*CY*CX
      VTY=VR*SY*SX+VE*CX-VN*CY*SX
      VTZ=VR*CY+VN*SY
      VX=VTX
      VY=VTY
      VZ=VTZ
      RETURN
      END

      SUBROUTINE V_GEOC_TO_LOCAL (VX,VY,VZ, VR,VE,VN)
C Utility * Real vector (components)
C
C The inverse of V_Local_to_Geoc
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON /CLOCAL/ X,Y,SX,CX,SY,CY,DUM2(6),DUMV(11),ISW
      VTR=VX*SY*CX+VY*SY*SX+VZ*CY
      VTE=-VX*SX+VY*CX
      VTN=-VX*CY*CX-VY*CY*SX+VZ*SY
      VR=VTR
      VE=VTE
      VN=VTN
      RETURN
      END

      SUBROUTINE LOCAL_TO_GEOC (Vin, Vout)
C Utility * Real vector
C
C Transform a local, complex displacement vector V_radial, V_east, V_north to
C geocentric representation. Local = at longitude X, colatitude Y (rad).
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VIN(3),VOUT(3),VT(3)
      COMMON /CLOCAL/ X,Y,SX,CX,SY,CY,DUM2(6),DUMV(11),ISW
      VT(1)=VIN(1)*SY*CX-VIN(2)*SX-VIN(3)*CY*CX
      VT(2)=VIN(1)*SY*SX+VIN(2)*CX-VIN(3)*CY*SX
      VT(3)=VIN(1)*CY+VIN(3)*SY
      VOUT(1)=VT(1)
      VOUT(2)=VT(2)
      VOUT(3)=VT(3)
      RETURN
      END

      SUBROUTINE GEOC_TO_LOCAL (Vin, Vout)
C Utility * Real vector
C
C The inverse of Local_to_Geoc
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VIN(3),VOUT(3),VT(3)
      COMMON /CLOCAL/ X,Y,SX,CX,SY,CY,DUM2(6),DUMV(11),ISW
      VT(1)=VIN(1)*SY*CX+VIN(2)*SY*SX+VIN(3)*CY
      VT(2)=-VIN(1)*SX+VIN(2)*CX
      VT(3)=-VIN(1)*CY*CX-VIN(2)*CY*SX+VIN(3)*SY
      VOUT(1)=VT(1)
      VOUT(2)=VT(2)
      VOUT(3)=VT(3)
      RETURN
      END

      SUBROUTINE Z_LOCAL_TO_GEOC (Vin, Vout)
C Utility * Complex vector
C
C Transform a local, complex displacement vector V_radial, V_east, V_north to
C geocentric representation. Local = at longitude X, colatitude Y (rad).
C
      IMPLICIT REAL*8 (A-H,O-U,X,Y), COMPLEX*16 (V)
      DIMENSION VIN(3),VOUT(3),VT(3)
      COMMON /CLOCAL/ X,Y,SX,CX,SY,CY,DUM2(6),DUMV(11),ISW
      VT(1)=VIN(1)*SY*CX-VIN(2)*SX-VIN(3)*CY*CX
      VT(2)=VIN(1)*SY*SX+VIN(2)*CX-VIN(3)*CY*SX
      VT(3)=VIN(1)*CY+VIN(3)*SY
      VOUT(1)=VT(1)
      VOUT(2)=VT(2)
      VOUT(3)=VT(3)
      RETURN
      END

      SUBROUTINE Z_GEOC_TO_LOCAL (Vin, Vout)
C Utility * Complex vector
C
C The inverse of Local_to_Geoc
C
      IMPLICIT REAL*8 (A-H,O-U,X,Y), COMPLEX*16 (V)
      DIMENSION VIN(3),VOUT(3),VT(3)
      COMMON /CLOCAL/ X,Y,SX,CX,SY,CY,DUM2(6),DUMV(11),ISW
      VT(1)=VIN(1)*SY*CX+VIN(2)*SY*SX+VIN(3)*CY
      VT(2)=-VIN(1)*SX+VIN(2)*CX
      VT(3)=-VIN(1)*CY*CX-VIN(2)*CY*SX+VIN(3)*SY
      VOUT(1)=VT(1)
      VOUT(2)=VT(2)
      VOUT(3)=VT(3)
      RETURN
      END

      SUBROUTINE SET_LOCAL_AT (XC,YC)
C Utility * Set site for local reference, unit sphere
c
c XC - longitude,  radians
c YC - colatitude, radians
c
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON /CLOCAL/ X, Y, SX, CX, SY, CY 
     &,               X1,Y1,SX1,CX1,SY1,CY1
     &,               V(3),W(3),B(3),R1,R2,ISW
      COMMON /MPAR/   PI,RAD
      X=XC
      Y=YC
      R1=1.D0
      SX=DSIN(XC)
      CX=DCOS(XC)
      SY=DSIN(YC)
      CY=DCOS(YC)
      V(1)=SY*CX
      V(2)=SY*SX
      V(3)=CY
      IF (ISW.EQ.1) THEN
         DO I=1,3
            B(I)=W(I)-V(I)
         ENDDO
      ENDIF
      PRINT '(A,2F10.4)',' <SLocAt>>> Long, colat ',XC/RAD,YC/RAD
      RETURN
      END

      SUBROUTINE SWAP_LOCAL (KSW)
C Utility * Swap local point / other point
C
C KSW<=0: always swap
C KSW =1: back to normal order
C KSW =2: switch to other order
C
      IMPLICIT   REAL*8 (A-H,O-Z)
      LOGICAL    QSWAP
      COMMON /CLOCAL/ X1(6)     ! point 1 on unit sphere
     &,               X2(6)     ! point 2 "    "    "
     &,               V(3),W(3),B(3),R1,R2,ISW     ! V1,V2, V2-V1, |V1|, |V2|
      IF (KSW.GT.2) THEN
         PRINT '(A,I5)',' <SwapLo>E> Bad argument:',KSW
         RETURN
      ENDIF
      QSWAP=KSW.LE.0.OR.(KSW.GT.0.AND.KSW-1.NE.ISW)
      IF (.NOT.QSWAP) RETURN
      ISW=MOD(ISW+1,2)
      DO I=1,3
         U=V(I)
         V(I)=W(I)
         W(I)=U
         B(I)=-B(I)
      ENDDO
      DO I=1,6
         U=X1(I)
         X1(I)=X2(I)
         X2(I)=U
      ENDDO
      RETURN
      END

      SUBROUTINE SET_BASELINE (V1,V2)
C Utility * Set reference baseline vector
      IMPLICIT REAL*8 (A-H,O-Z)
      LOGICAL QE
      DIMENSION  V1(3),V2(3)
      COMMON /CLOCAL/ X  ,Y  ,SX  ,CX ,SY ,CY      ! point 1 on unit sphere
     &,               X1, Y1, SX1, CX1,SY1,CY1     ! point 2 "    "    "
     &,               V(3),W(3),B(3),R1,R2         ! V1,V2, V2-V1, |V1|, |V2|
     &,               ISW                          ! swap-id

      goto 10

      ENTRY SET_V_BASELINE (V11,V12,V13, V21,V22,V23)
      V1(1)=V11
      V1(2)=V12
      V1(3)=V13
      V2(1)=V21
      V2(2)=V22
      V2(3)=V23
      
 10   QE=.FALSE.
      DO I=1,3
         V(I)=V1(I)
         W(I)=V2(I)
         B(I)=V2(I)-V1(I)
      ENDDO
      GOTO 1

      ENTRY SET_LOCAL_AT_XYZ (V1)
      GOTO 11

      ENTRY SET_LOCAL_V_XYZ (V11,V12,V13)
      V1(1)=V11
      V1(2)=V12
      V1(3)=V13
      
C Utility * Set site for local reference, XYZ vector
 11   DO I=1,3
         V(I)=V1(I)
      ENDDO
      QE=.TRUE.

1     CALL XYZ_TO_GEOC (V1(1),V1(2),V1(3),X,Y)
      SX=DSIN(X)
      CX=DCOS(X)
      SY=DSIN(Y)
      CY=DCOS(Y)
      R1=V_2_NORM(V1)
      IF (QE) RETURN
      
      CALL XYZ_TO_GEOC (V2(1),V2(2),V2(3),X1,Y1)
      SX1=DSIN(X1)
      CX1=DCOS(X1)
      SY1=DSIN(Y1)
      CY1=DCOS(Y1)
      R2=V_2_NORM(V2)

      RETURN
      END

      SUBROUTINE CHORD_TO (XC,YC)
C Utility * Set site for local reference
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON /CLOCAL/ X0, Y0, SX0, CX0, SY0, CY0
     &,               X,  Y,  SX,  CX,  SY,  CY
     &,               V(3),W(3),BL(3),R1,R2,ISW
      COMMON /MPAR/   PI,RAD
      ENTRY SET_CHORD_TO (XC,YC)
      X=XC
      Y=YC
      SX=DSIN(XC)
      CX=DCOS(XC)
      SY=DSIN(YC)
      CY=DCOS(YC)
      BL(1)=SY*CX-SY0*CX0
      BL(2)=SY*SX-SY0*SX0
      BL(3)=CY-CY0
      DO I=1,3
         W(I)=1.D0
      ENDDO
      R2=1.D0
      PRINT '(A,2F10.4)',' <ChorTo>>> Long, colat ',XC/RAD,YC/RAD
      PRINT '(A,1P,3D20.10)',' <ChorTo>>> Baseline Vector=',BL
      RETURN
      END

      SUBROUTINE SET_LTU_ROT
C Utility * Set LTU transformation matrix at local point
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON /CLOCAL/ X0, Y0, SX0, CX0, SY0, CY0
     &,               X,  Y,  SX,  CX,  SY,  CY
     &,               V(3),W(3),B(3),R1,R2,ISW
      COMMON /CROTMA/ CM
      DIMENSION CM(3,3),BUV(3),TUV(3),UUV(3),BT(3),UV(3),ZUV(3)
      DATA ZUV/0.0d0,0.0d0,1.0d0/

      ENTRY MAKE_LTU_ROT
C Utility * alias
      CALL UNIT_V (B,BUV)               ! buv = baseline unit vector
      CALL CROSS_PRD (V,B,BT)           ! bt = right angle on r1-b plane
      CALL UNIT_V (BT,TUV)              ! tuv = transverse unit vector
      CALL CROSS_PRD (BUV,TUV,UUV)      ! buv x tuv = up unit vector

      DO 101 I=1,3
      CM(1,I)=BUV(I)
      CM(2,I)=TUV(I)
      CM(3,I)=UUV(I)
101   CONTINUE

      CALL SHOW_ROT ('LTU-rotation matrix:')
      RETURN

      ENTRY GET_ROT_COMP (IC,UV)
      DO I=1,3
      UV(I)=CM(IC,I)
      ENDDO
      RETURN

      ENTRY SET_REN_ROT
C Utility * Another orientation: Radial, East, North at local point
      CALL UNIT_V (V,UUV)
      CALL CROSS_PRD (ZUV,UUV,BT)
      CALL UNIT_V (BT,TUV)
      CALL CROSS_PRD (UUV,TUV,BUV)
      DO I=1,3
         CM(1,I)=UUV(I)
         CM(2,I)=TUV(I)
         CM(3,I)=BUV(I)
      ENDDO

      CALL SHOW_ROT ('REN-rotation matrix:')
      RETURN
      END

      SUBROUTINE XYZ_TO_SPHERE (X,XLO,XCL)
      IMPLICIT REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      COMMON /MPAR/ PI,RAD
      DIMENSION X(3)
      Z=CMPLX(X(1),X(2))
      A=ABS(Z)
      XLO=DFATG(Z)
      XCL=PI/2.D0-DATAN(X(3)/A)
      RETURN
      END
      
      
