C This file contains the subprograms for strain tides computation.
C
C     ---------------------------------
      SUBROUTINE Strain_Earth (X,Y,ZDR)
C BTIDE Strain * Elastic elliptic earth model
C     ---------------------------------
C>
C After CALL ASTRO(...), Strain_Earth can be called to return ZDR(1200,3).
C
C ENTRY          Strain_Anela_Earth (X,Y,ZDR)
C incorporates anelasticity by multiplication of the Basic Love numbers
C with anelastic relaxation factor.
C
C ENTRY          D_Strain_D_Phi (X,Y,ZDR)
C returns the strain spectrum derivative w r t the phase parameter of the
C anelasticity model.
C
C ZDR(i,k) - complex*16 - is the tide strain spectrum for a station at...
C X        - real*8     - east longitude and...
C Y        - real*8     - colatitude
C                         (geocentric coordinates, units = radians).
C
C i   = frequency index, i=0..NV. NV is found in common /CASTSP/.
C k   = component index (NS, EW, NE for k=1..3).
C
C In the strain tide spectrum, the Basic Love numbers are already applied.
c
c ZDR should be initialized as  3600*(0.0d0,0.0d0) to avoid problems
c with eventual NaN's.
C
C Earth model: Elastic, elliptical earth with liquid core, WAHR (1981),
C refined core resonance parameters according to NEUBERG et al. (1987).
C The coefficients of the leading terms turn out practically
C identical to the Love numbers:
C S1 = h, S2 = l; only S3 = WS3(/WAHRP/) is not equal Wahr's y
C
C
C ENTRY      D_Strain_Earth (X,Y,ZDR,ZO,ZM)
C
C Returns a linear combination   ZO*ZDR.in + ZM*ZDR.new = ZDR.out
C supposedly after an earth or site parameter has been changed. This call
C provides a way to obtain derivatives w r t a parameter.
C
C
C In order to obtain tidal strain of component k at time T, perform
C
C   eps(k) = Real ( Sum (i=1,NV)  ZDR(i,k) * Exp(ZI*FRE(i)*T) )
C
C where
C       k = component, 1 - NS, 2 - EW, 3 - NE
C      ZI = imaginary unit
C     FRE = Frequency array, found in common /CASTSP/
C      NV = length of spectrum, found in common /CASTSP/
C       T = time from initial epoch in hours.
C More help ...
C_
C In order to obtain tidal strain at azimuth A (clockwise from north),
C evaluate:
C
C   eps(A) = eps(NS)*cos^2(A) + eps(EW)*sin^2(A) + 2*eps(NE)*cos(A)*sin(A)
C
C which differs from Wahr's definition in eps(NE).
c
c Multiple calls to Strain_Earth, too much printing; print option is turned
c off automatically after the first call.
c
c Call  Strain_Earth_Msg  to set print option
C
      IMPLICIT        REAL*8 (A-H,O-Y), COMPLEX*16 (Z)
      INTEGER*2       IAA,IATR(0:9)
      INTEGER         ICO(2:4,0:4)
      REAL*8          P(2:4,0:4), DP(2:4,0:4), DDP(2:4,0:4)
     &,               CORDH(2:4,0:4), CORDL(2:4,0:4), CORDK(2:4,0:4)
      REAL*8          WLP0(0:2), WWP0(0:2), WZ0(0:2)
     &,               WKP0(0:2), WSLP(0:2)
     &,               WTS(0:3),WTE(0:2)
      COMPLEX*16      ZLO(0:4), ZDR(1200,3), ZBL(3)
      LOGICAL         QANEL,QDANEL,qp
C
C Tide potential spectrum:
      COMMON /CASTSP/ AMP(1200),PHA(1200),FRE(1200),NV,IAA(0:9,1200)
C
C Maths:
      COMMON /MPAR/   PI,RAD
C
C Earth parameters:   Mean radius, equat.radius, equat.gravity, rot.velocity,
C                     Basic Love numbers:
      COMMON /EPAR/   R0,RE,GE,EROT,BL(3),GM,GRVLAT,ESQUAR,FE
C
C Near diurnal resonance (NDR) parameters:
      COMMON /COREP/  ONDR,QNDR,FREFC
     &,               SEHNDR,SELNDR,SEKNDR,ZOHNDR,ZOLNDR,ZOKNDR
C
C Love number ratios for spherical harmonic degrees and orders w r t  n=2, m=1
      COMMON /SHDOP/  CORDH, CORDL, CORDK
C
C Spherical harmonic coupling parameters; Wahr-model
      COMMON /WAHRP/  WWP0,WLP0,WZ0,WY,WWP1,WLP1,WZ1,WS3
     &,               WKP0, WSLP
     &,               WTS,WTE
C
      DATA            ZI/(0.0D0,1.0D0)/
     &,     IATR/2,2,0,0,0,0,0,0,0,0/
     &,     qp/.true./
      QANEL=.false.
      QDANEL=.FALSE.
      GOTO 2
C
      ENTRY Strain_Anela_Earth (X,Y,ZDR)
C BTIDE Strain * ... + anelastic earth
C
      QANEL=.true.
      QDANEL=.false.
      GOTO 2
C
      ENTRY D_Strain_D_Phi (X,Y,ZDR)
C BTIDE Strain * Derivative w.r.t. phase (anelast.) parameter
C
      QANEL=.true.
      QDANEL=.true.
2     ZO=(0.0D0,0.0D0)
      ZM=(1.0D0,0.0D0)
      GOTO 1
C
      ENTRY D_Strain_Earth (X,Y,ZDR,ZOD,ZMD)
C BTIDE Strain * Lin.comb. after parameter redefinition
C
      ZO=ZOD
      ZM=ZMD
C
    1 IF (Y/RAD.LT.1.D-3.OR.Y/RAD.GT.179.99D0) THEN
         PRINT '(A,F10.4)','0<STRAIN>W> Invalid colatitude :'
     &,   Y/RAD,'. Adjusted.'
         Y=DMIN1(179.99*RAD,DMAX1(1.D-3*RAD,Y))
      ENDIF
      FRENDR=(1.D0+1.D0/ONDR)*EROT
      ZFRNDR=CMPLX(FRENDR,FRENDR/QNDR/2.D0)
C                                       (N.B. use DCMPLX if available)
      TNSMIN=1.D10
      TEWMIN=1.D10
      TNEMIN=1.D10
C
C Spherical harmonics for the site:
      if (qp) then
         PRINT '(A,2F12.6)'
     &,   '0<STRAIN>>> Site longitude / colatitude:',X/RAD,Y/RAD
     &,   ' <STRAIN>>> Assigning columns 1..3: eps_NS, eps_EW, eps_NE'
         PRINT '(A)',' <STRAIN>>>'
     &,              ' <STRAIN>>>  N       P(N,M), DP(N,M)'
     &,              ' <STRAIN>>>        DDP(N,M)           M=1..N'
      endif
C
      SINY=DSIN(Y)
      COSY=DCOS(Y)
      CTGY=COSY/SINY
C
      DO 20 N=2,4
      DNN1=DFLOAT(N*(N+1))
      DO 21 M=0,N
      DM=DFLOAT(M)
      ICO(N,M)=0
      P(N,M)=PNM(N,M,Y)
      DP(N,M)=DPNM(N,M,Y)
      DDP(N,M)=(DM*DM/SINY/SINY-DNN1)*P(N,M)-CTGY*DP(N,M)
21    CONTINUE

      if (qp) then
         print 1003
         PRINT 1001, N,(P(N,M),DP(N,M),M=0,N)
         PRINT 1002,(DDP(N,M),M=0,N)
      endif
20    CONTINUE
      RS=R0*(1.D0-2.D0/3.D0*0.00334D0*P(2,0))
      if (qp) PRINT 1003
      DO 22 M=0,4
      DM=DFLOAT(M)
22    ZLO(M)=EXP(ZI*DM*X)

1001  format (' <STRAIN>>>',I3,5(2X,2F9.4))
1002  format (' <STRAIN>>>',3X,5(2X,F9.4,9X))
1003  format (' <STRAIN>>>')
C                          (N.B. use CDEXP instead of EXP if available)
C
C ------------------------- Step through the astron.spectrum -->>>
      DO 101 IV=1,NV
      N=IAA(0,IV)
      DNN1=DFLOAT(N*N+N)
C
C Discard sph.harm.degree > 4
      IF (N.GT.4) THEN
         ZDR(IV,1)=(0.0D0,0.0D0)
         ZDR(IV,2)=(0.0D0,0.0D0)
         ZDR(IV,3)=(0.0D0,0.0D0)
         GOTO 101
      ENDIF
      M=IAA(1,IV)
      ICO(N,M)=ICO(N,M)+1
      DM=DFLOAT(M)
      IF (QDANEL) THEN
         ZBL(1)=(0.0D0,0.0D0)
         ZBL(2)=(0.0D0,0.0D0)
      ELSE
         ZBL(1)=BL(1)
         ZBL(2)=BL(2)
      ENDIF
C
      IF (N.EQ.2) THEN
C
C Leading terms
C
         IF (M.EQ.1) THEN
C
C Liquid core effects
C
            ZCLQCH=SEHNDR*(FRE(IV)-FREFC)/(FRE(IV)-ZFRNDR)
            ZCLQCL=SELNDR*(FRE(IV)-FREFC)/(FRE(IV)-ZFRNDR)
            ZOLQCH=ZOHNDR*EROT/(FRE(IV)-ZFRNDR)
            ZOLQCL=ZOLNDR*EROT/(FRE(IV)-ZFRNDR)
C
         ELSE
C
C Long period or semidiurnal
C
            ZCLQCH=(0.0D0,0.0D0)
            ZCLQCL=(0.0D0,0.0D0)
            ZOLQCH=(0.0D0,0.0D0)
            ZOLQCL=(0.0D0,0.0D0)
         ENDIF
C
         IF (QANEL) THEN
            Z0=ZANEL(0,FRE(IV),ZD0)
            Z1=ZANEL(1,FRE(IV),ZD1)
            IF (QDANEL) THEN
               ZBL(1)=BL(1)*ZD0
               ZBL(2)=BL(2)*ZD1
            ELSE
               ZBL(1)=BL(1)*Z0
               ZBL(2)=BL(2)*Z1
            ENDIF
         ENDIF
         S3U=WS3
      ELSE
C
C Spherical harmonic degree > 2
         ZCLQCH=(0.0D0,0.0D0)
         ZCLQCL=(0.0D0,0.0D0)
         ZOLQCH=(0.0D0,0.0D0)
         ZOLQCL=(0.0D0,0.0D0)
         S3U=0.0D0
C
      ENDIF
C
C Put together: potential ampl. / g / r * (spherical eff. + ellips.perturb.)
      XAMP=AMP(IV)/GE/RS
      ZNS= XAMP*( (1.D0+ZCLQCH+ZOLQCH) * ZBL(1)*CORDH(N,M)*P(N,M)
     &   +        (1.D0+ZCLQCH)        * S3U*P(2,0)*DDP(N,M)
     &   + ZBL(2)*CORDL(N,M)*(1.D0+ZCLQCL+ZOLQCL) * DDP(N,M) )

c      print *,'NN:',iv,':: ',zns,xamp,CORDH(N,M),ZCLQCH,ZOLQCH
c     &,        CORDL(N,M),ZCLQCL,ZOLQCL,zbl(1),zbl(2),s3u

      ZEW= XAMP*(
     &      ZBL(1)*CORDH(N,M)*(1.D0+ZCLQCH+ZOLQCH) * P(N,M)
     &   + (ZBL(2)*CORDL(N,M)*(1.D0+ZCLQCL+ZOLQCL)
     &      +      S3U*P(2,0)*(1.D0+ZCLQCH) )
     &   * ( CTGY*DP(N,M)-DM*DM/SINY/SINY*P(N,M) )  )
c
c Obs! Wahr's definition of Z_NE is actually 2*Z_SouthEast
c
      ZNE=-XAMP*ZI*(
     &     ZBL(2)*CORDL(N,M)*(1.D0+ZCLQCL+ZOLQCL)
     &     +      S3U*P(2,0)*(1.D0+ZCLQCH)        )
     &   * ( DP(N,M)-CTGY*P(N,M) ) * DM/SINY
C
      ZPHA=EXP(ZI*PHA(IV))
C                          (N.B. use CDEXP instead of EXP if available)
c      print *,iv,':: ',ZNS,ZLO(M),ZPHA,ZM,ZO,ZDR(IV,1)
      ZDR(IV,1) = ZNS*ZLO(M)*ZPHA*ZM + ZO*ZDR(IV,1)
      ZDR(IV,2) = ZEW*ZLO(M)*ZPHA*ZM + ZO*ZDR(IV,2)
      ZDR(IV,3) = ZNE*ZLO(M)*ZPHA*ZM + ZO*ZDR(IV,3)
C
c      QTR=.TRUE.
c      DO 901 I=0,6
c      QTR=QTR.AND.IAA(I,IV).EQ.IATR(I)
c901   CONTINUE
c      IF (QTR) THEN
c         PRINT '(3H0Tr,2F10.4,1P,D12.4/(3X,1P,2D12.4))'
c     &,  AMP(IV),PHA(IV)/RAD,XAMP,ZNS,ZLO(M),ZPHA,ZM,ZO
c      ENDIF
C
C Check smallest amplitude (N.B. use CDABS if available):
      ANS=ABS(ZDR(IV,1))
      AEW=ABS(ZDR(IV,2))
      ANE=ABS(ZDR(IV,3))
      TNSMIN=DMIN1(ANS,TNSMIN)
      TEWMIN=DMIN1(AEW,TEWMIN)
      TNEMIN=DMIN1(ANE,TNEMIN)
C ------------------------------------------------ End loop ---<<<
101   CONTINUE

      if (qp) then
         PRINT '(A,1P,3D10.2)'
     &,   ' <STRAIN>>> Min. tide amplitudes (NS, EW, NE) :'
     &,   TNSMIN, TEWMIN, TNEMIN
         PRINT '(A)',' <STRAIN>>>'
     &,   ' <STRAIN>>> Number of tides in the different bands:'
     &,   ' <STRAIN>>>  N / M=0    1    2    3    4'
         DO 111 N=2,4
111      PRINT 1004, N,(ICO(N,M),M=0,N)
         PRINT 1005,' BLMULT-option turned off.'
cc      else
cc         print 1005,' Ready.'
      endif
      qp=.false.
      CALL BLMULT ('NO ')
      RETURN
1004  format (' <STRAIN>>> ',I2,1X,5I5)
1005  format (' <STRAIN>>>',A)

      entry strain_earth_msg
c BTIDE Strain * Revive message option
c 
      qp=.true.
      return
      END

